//com.h
/* Libreria que contiene las funciones basicas para realizar
una comunicacion via radio entre el PC y distintos Microbots */

/*----------------------Librerias incluidas--------------------*/

#include <time.h>
#include <dos.h>

/*----------------------Definicion de constantes---------------*/

#define TRUE    1
#define FALSE   0
#define PIC     0x20    /* Programmable Interrupt Controller (PIC) Base Address */
#define MASK    PIC+1
#define IRQ     7       /* interrupcion del Puerto Paralelo */
#define CLK_TCK 18.2
#define TETX    0.17    /* Tiempo de espera en transmision */
#define TERX    0.25    /* Tiempo de espera en recepcion */

/*----------------------Variables globales----------------------*/

unsigned short int transmitiendo;      /* Variable que indica si se esta transmitiendo */
unsigned short int dato_recibido;      /* variable que indica que se ha recibido un dato */
unsigned short int dir_local = 0;      /* direccion local del PC */
unsigned int DATA = 0x378;             /* Inicializacion de las dir. del PP con los valores standard */
unsigned int STATUS = 0X379;
unsigned int CONTROL = 0X37A;
unsigned short int intlev = IRQ+0x08;  /* 0x0F */
void interrupt far (*oldhandler)();

/*----------------------Funciones--------------------------------*/

unsigned short int enviar(unsigned short int dir,unsigned short int dato);
unsigned short int recibir();
unsigned short int datoListo();
void inicializar();
void terminar();
void setDirLocal(unsigned short int dir);
unsigned short int getDirLocal();
unsigned short int enviarNibble(unsigned short int dir, unsigned short int comando);
unsigned short int recibirNibble(unsigned short int *dato);
void interrupt far intserv();     /* Interrupt Service Routine (ISR) */

/*----------------------Procedimiento enviar---------------------*/

unsigned short int enviar(unsigned short int dir, unsigned short int dato)
{
    clock_t t1,t2;
    t1=clock();
    while ((transmitiendo==TRUE) || (inportb(STATUS) & 0x40))
    {
	t2=clock();
	if ( ((t2-t1)/CLK_TCK) > TETX ) {
		return(FALSE);  // Retorna que no se ha podido comenzar la transmision
	}
    }
    transmitiendo=TRUE;                       // indica que se esta transmitiendo
    outportb(DATA,(dir<<4) | (dato & 0x0F));  // configura el modem con la direccion local del PC
    outportb(CONTROL,0x11);     // pone el modem en TX e inicia la transmision activando /TE
    delay(0.0001);              // espera tw
    outportb(CONTROL,0x10);     // desactiva /TE
    outportb(MASK,inportb(MASK) & ~0x80);  // habilita la interrupcion del PP
    return(TRUE);               // Retorna que ha comenzado la transmision correctamente
}

/*----------------------Funcion recibir---------------------------*/

unsigned short int recibir()    // Lee el dato recibido
{
    unsigned short int dato,temp;
    dato=inportb(STATUS);                  // Lee el dato del registro STATUS del PP
    dato=dato >> 3;
    temp=~(dato | ~0x10);
    temp=temp >> 1;
    dato=(dato & 0x07);
    dato=(dato | temp);
    dato_recibido=FALSE;		   // Indica que se ha leido el dato recibido
    outportb(CONTROL,0x12);		   // Configura el modem en Recepcion
    outportb(MASK,inportb(MASK) & ~0x80);  // habilita la interrupcion del PP
    return(dato);                          // Retorna el dato leido
}

/*----------------------Funcion dato listo-------------------------*/

unsigned short int datoListo()
{
	return(dato_recibido);  // Retorna si se ha recibido algun dato
}

/*----------------------Procedimiento inicializar------------------*/

void inicializar()   // Inicializa el Modem
{
    unsigned int far *xptr;
    xptr=(unsigned int far *)0x00000408;
    DATA=*xptr;                            // Establece las direcciones del PP para
    STATUS=DATA+1;                         // para la maquina en particular
    CONTROL=DATA+2;                        // a partir de la tabla de Look Up que proporciona la BIOS
    dato_recibido=FALSE;
    transmitiendo=FALSE;
    outportb(CONTROL,0x12);		   // habilita la interrupcion del PP por ACK y pone el modem en RX
    outportb( DATA , dir_local<<4 );
    oldhandler = getvect(intlev);          // salva el anterior vector de interrupcion
    setvect(intlev, intserv); 		   // direcciona la rutina de atencion a la interrupcion
    outportb(MASK,inportb(MASK) & ~0x80);  // habilita la interrupcion del PP
}

/*----------------------Procedimiento terminar-----------------------*/

void terminar()   // restaura el sistema a sus valores iniciales
{
    setvect(intlev, oldhandler);		 // restaura el anterior vector de interrupcion
    outportb(MASK,inportb(MASK) | 0x80);         // deshabilita la interrupcion del PP
    outportb(CONTROL,0x02);                      // deshabilita la interrupcion del PP por ACK
}

/*----------------Procedimiento cambiar direccion local--------------*/

void setDirLocal(unsigned short int dir)
{
	dir_local=dir;      // Establece una nueva direccion local del PC
	outportb( DATA , (inportb(DATA) & 0x0F) | (dir_local<<4) );
}

/*----------------Procedimiento obtener direccion local--------------*/

unsigned short int getDirLocal()
{
	return(dir_local);   // Retorna la actual direccion local del PC
}

/*----------------------Funcion enviar nibble----------------------*/

unsigned short int enviarNibble(unsigned short int dir, unsigned short int comando)
{                  // Funcion que transmite un dato asegurandose de que la transmision se reliza correctamente
    clock_t t1,t2;
    if ( enviar(dir,comando)==FALSE ) {
	transmitiendo=FALSE;
	dato_recibido=FALSE;
	outportb( DATA , (inportb(DATA) & 0x0F) | (dir_local<<4) ); // Configura el modem con la direccion local del PC
	outportb(CONTROL,0x12);                                     // Configura el modem en recepcion
	return(FALSE);    // Retorna que no se ha completado la transmision satisfactoriamente
    }
    t1=clock();
    t2=t1;
    while ( (transmitiendo==TRUE) && ( ((t2-t1)/CLK_TCK) < TETX ) )
    {
	t2=clock();
    }
    if (transmitiendo==TRUE) {
	transmitiendo=FALSE;
	dato_recibido=FALSE;
	outportb( DATA , (inportb(DATA) & 0x0F) | (dir_local<<4) ); // Configura el modem con la direccion local del PC
	outportb(CONTROL,0x12);
	return(FALSE);    // Retorna que no se ha completado la transmision satisfactoriamente
    } else if ( comando==recibir() ) {  // Lee el dato del modem para comprobar que se haya transmitido correctamente
	return(TRUE);     // Retorna que se ha completado la transmision satisfactoriamente
    } else {
	return(FALSE);    // Retorna que no se ha completado la transmision satisfactoriamente
    }
}

/*----------------------Funcion recibir nibble---------------------*/

unsigned short int recibirNibble(unsigned short int *dato)
{                  // Funcion que espera un tiempo TERX al recepcion de un dato
    clock_t t1,t2;
    t1=clock();
    t2=t1;
    outportb(CONTROL,0x12);	// se asegura que el modem esta en Recepcion
    while ( (dato_recibido==FALSE) && ( ((t2-t1)/CLK_TCK) < TERX ) )
    {
	t2=clock();
    }
    if (dato_recibido==TRUE) {
	*dato = recibir();  // Lee el dato recibido
	return(TRUE);       // Retorna que se ha recibido un dato antes de transcurrido TERX
    } else {
	return(FALSE);      // Retorna que no se ha recibido un dato antes de transcurrido TERX
    }
}

/*----------------Rutina de atencion a la interrupcion del PP--------*/

void interrupt far intserv()
{
    outportb(MASK,inportb(MASK) | 0x80);	// deshabilita la interrupcion del PP
    outportb(PIC,0x20);  			// indica que la interrupcion ha sido procesada
    if (transmitiendo==TRUE) {                  // comprueba si se estaba o no transmitiendo
	transmitiendo=FALSE;                    // indica que se ha terminado de transmitir
	dato_recibido=FALSE;
	outportb( DATA , (inportb(DATA) & 0x0F) | (dir_local<<4) );  // Configura el modem con la direccion local del PC
	outportb(CONTROL,0x12);                 // configura el modem en recepcion
	outportb(MASK,inportb(MASK) & ~0x80);   // habilita la interrupcion del PP
    }
    else {
	outportb(CONTROL,0x10);		        // pone el puerto en TX para no machacar el dato recibido
	dato_recibido=TRUE;                     // indica que se ha recibido un dato
    }
}