;ECO.ASM
;Programa que hace ECO del dato recibido

OPTION  EQU $39
PIOC    EQU $02
PORTA   EQU $00
PORTD   EQU $08
DDRD    EQU $09
PORTE   EQU $0A
SCCR2   EQU $2D
SPCR	EQU $28
HPRIO   EQU $3C
TMSK1   EQU $22
TMSK2   EQU $24
TFLG1   EQU $23
TCTL1   EQU $20
DIRPC   EQU $00
TOC4    EQU $1C
TCNT    EQU $0E	
TIEMPO  EQU 65000  ; Numero de pulsos de reloj necesarios para generar un
                   ; retraso 32,5 ms. Cada tic de reloj son 0.5 us
                   ; 65000*8 = 32500 us = 32,5 ms.

        ORG $0000	; RAM

        BRA config
	
;variables

dato_recibido  DB 0
dato           DB 0
dir            DB 0
cuenta         DB 0

config	LDX #$1000
        LDS #$00B4              ; PILA en DIR 180

	LDAA #$00
        STAA  SCCR2,X           ; SCI deshabilitado
	
        LDAA #$30
        STAA  SPCR,X            ; SPI deshabilitado y salidas OC
	
        LDAA  #$30              ; BITS 0 a 3 del puerto D entradas,
        STAA  DDRD,X            ; BITS 4 y 5 del puerto D salidas
        
        LDAA  #$20
        STAA  PORTD,X           ; BIT 4 a 0 y BIT 5 del puerto D a 1
        
        BSET OPTION,X $20       ; IRQ activa por flanco de bajada

        LDAA #$C6
        STAA HPRIO,X            ;Maxima prioridad IRQ

        LDAA #$00
        STAA TMSK1,X            ; No permitir la interupcin de los comparadores

        LDAA #$00
        STAA dato_recibido
        
        CLI                     ; permite interrupciones

        LDAA #DIRPC               
bucle   LDAB dato_recibido      ;espera a recibir un dato
        CMPB #$00
        BEQ bucle
        BSR recibir             ;lee el dato recibido
        BSR enviar              ;envia el dato recibido
        BRA bucle

;Funcion enviar: Transmite el dato en B a la direccion en A

enviar  STAA dir
        STAB dato
ex      BRSET PORTD,X $10 ex ;espera a que se termine la anterior transmision
	
        BSR delay            ;Espera 32,5 ms

        BSET DDRD,X $0F      ;BITS 0 a 3 del puerto D salidas

        LDAA dir
        ANDA #$0F
        ORAA #$10
        STAA PORTD,X         ;Escribe la direccion destino e inicia la transmision
        
        BSET PORTD,X $20
        BSET PORTD,X $20
        BSET PORTD,X $20
        BSET PORTD,X $20     ;Se repite para que transcurra thl

        LDAB dato
        ORAB #$30
        STAB PORTD,X         ;Escribe el dato a transmitir

        ANDA #$0F

	RTS

;Funcion recibir: lee de la tarjeta el dato recibido

recibir BRSET PORTD,X $10 recibir  ;espera a que no se este transmitiendo
        LDAB #$00
        STAB dato_recibido         ;indica que se ha leido el dato
        ;BCLR DDRD,X $0F
        LDAB PORTD,X               ;lee el dato y lo deja en el acum B
        ANDB #$0F
        RTS

;Funcion delay: genera un retardo de 32,5 ms

delay   LDD TCNT,X                 ;Subrutina que espera 32,5 ms
        ADDD #TIEMPO               ;Esperar 32,5 ms
        STD TOC4,X                 ;Escribe en el comparador
        BSET TFLG1,X $10           ;Pone a cero flag del comparador 4
buclees BRCLR TFLG1,X $10 buclees  ;Comprobar si a terminado la cuenta
        RTS

;Rutina de atencion a la interrupcion externa IRQ

int_irq BRSET PORTD,X $10 fin_tr  ;Si se estaba transmitiendo salta a finalizar la transmision
        LDAA #$01
        STAA dato_recibido        ;Si no se estaba transmitiendo activa dato_recibido
        RTI
fin_tr  BCLR DDRD,X $0F           ;BITS 0 a 3 del puerto D entradas
        BCLR PORTD,X $10          ;Pone la tarjeta en recepcion
        RTI

        ORG $00EE
        JMP int_irq
        
	END