//interfaz.c
/* Programa que muestra un panel desde donde controlar los Microbots */

/*--------------------Librerias incluidas----------------*/

#include <math.h>
#include "protocol.h"
#include "panel.h"
#include "userint.h"

/*--------------------Definicion de Constantes-----------*/

#define DIMGRAPH 200        //Dimension de la grafica
#define MODULO   1          //Distancia a avanzar en la grafica por cada paso de rueda
#define PI       3.14
#define P2PI     32         //Pasos de rueda necesarios para girar 360 grados
#define ALPHA    2*PI/P2PI  // angulo de giro por cada paso en radianes
#define ALPHAG   360.0/P2PI // angulo de giro por cada paso en grados

/*--------------------Variables globales------------------*/

int panel_control;
double x = DIMGRAPH/2.0;    // coordenada x incial del robot en la grafica
double y = DIMGRAPH/2.0;    // coordenada y incial del robot en la grafica
short int dir = 0;          // rumbo inicial del microbot

/*--------------------Declaracion de funciones------------*/

void dibujarPunto(short int sentido, unsigned short int pasos);
void girarPunto(short int sentido, unsigned short int pasos);

/*--------------------Programa principal------------------*/

void main()
{
   int controla;
   unsigned short int robot = 15;
   unsigned short int numPasos = 0;
   unsigned short int temp,temp2,temp3,dato;

   panel_control = LoadPanel("panel.uir",MAIN);  /* Cargamos el panel */
   DisplayPanel(panel_control);                  /* Mostramos el panel */
   inicializar();                                /* Inicializamos el Modem */
   SetCtrlVal(panel_control, MAIN_DIRLOCAL, getDirLocal()); /* Inicializamos indicador de Dir Local */

   DeletePlots (panel_control, MAIN_GRAPH);      /* Limpiamos la grafica */
   PlotPoint (panel_control, MAIN_GRAPH, x, y, 0, 1);  /* Dibujamos posicion del robot en la grafica */
   SetCtrlVal(panel_control, MAIN_RUMBO, ALPHAG*dir);  /* Se inicializa el rumbo del robot */

   MessagePopup("Pulse SALIR para terminar");

   while(TRUE)
       {
       while(GetUserEvent(0, &panel_control, &controla) == 0) {  /*Esperamos a que se utilce un control en el panel principal*/
	      if(datoListo()==TRUE) {
			recibir();
			SetCtrlVal(panel_control, MAIN_CONSOLA, "Dato recibido.");
	      }
       }
       GetCtrlVal(panel_control, MAIN_ROBOT, &robot);
       switch(controla) {
	case MAIN_SALIR:       /* el usuario ha pulsado salir */
	  terminar();
	  UnloadPanel (panel_control);
	  return;
	case MAIN_DELANTE:     /* avanzar */
	  GetCtrlVal(panel_control, MAIN_NUMPASOS, &numPasos);
	  SetCtrlVal(panel_control, MAIN_CONSOLA, "Enviando comando AVANZAR.");
	  temp=avanzar(robot,numPasos);
	  if (temp==2) {
	       MessagePopup("Fallo en la conexion con el modem.");
	  } else if (temp==1) {
	       SetCtrlVal(panel_control, MAIN_CONSOLA, "Imposible conectar con el robot.");
	  } else {
	       SetCtrlVal(panel_control, MAIN_CONSOLA, "Avanzando.");
	       dibujarPunto(1,numPasos);
	  }
	  break;
	case MAIN_ATRAS:      /* retroceder */
	  GetCtrlVal(panel_control, MAIN_NUMPASOS, &numPasos);
	  SetCtrlVal(panel_control, MAIN_CONSOLA, "Enviando comando RETROCEDER.");
	  temp=retroceder(robot,numPasos);
	  if (temp==2) {
	       MessagePopup("Fallo en la conexion con el modem.");
	  } else if (temp==1) {
	       SetCtrlVal(panel_control, MAIN_CONSOLA, "Imposible conectar con el robot.");
	  } else {
	       SetCtrlVal(panel_control, MAIN_CONSOLA, "Retrocediendo.");
	       dibujarPunto(-1,numPasos);
	  }
	  break;
	case MAIN_DERECHA:     /* girar a la derecha */
	  GetCtrlVal(panel_control, MAIN_NUMPASOS, &numPasos);
	  SetCtrlVal(panel_control, MAIN_CONSOLA, "Enviando comando GIRAR DERECHA.");
	  temp=girarDerecha(robot,numPasos);
	  if (temp==2) {
	       MessagePopup("Fallo en la conexion con el modem.");
	  } else if (temp==1) {
	       SetCtrlVal(panel_control, MAIN_CONSOLA, "Imposible conectar con el robot.");
	  } else {
	       SetCtrlVal(panel_control, MAIN_CONSOLA, "Girando a la derecha.");
	       girarPunto(1,numPasos);
	  }
	  break;
	case MAIN_IZQUIERDA:    /* girar a la izquierda */
	  GetCtrlVal(panel_control, MAIN_NUMPASOS, &numPasos);
	  SetCtrlVal(panel_control, MAIN_CONSOLA, "Enviando comando GIRAR IZQUIERDA.");
	  temp=girarIzquierda(robot,numPasos);
	  if (temp==2) {
	       MessagePopup("Fallo en la conexion con el modem.");
	  } else if (temp==1) {
	       SetCtrlVal(panel_control, MAIN_CONSOLA, "Imposible conectar con el robot.");
	  } else {
	       SetCtrlVal(panel_control, MAIN_CONSOLA, "Girando a la izquierda.");
	       girarPunto(-1,numPasos);
	  }
	  break;
	case MAIN_STOP:         /* parar */
	  SetCtrlVal(panel_control, MAIN_CONSOLA, "Enviando comando PARAR.");
	  temp=parar(robot);
	  if (temp==2) {
	       MessagePopup("Fallo en la conexion con el modem.");
	  } else if (temp==1) {
	       SetCtrlVal(panel_control, MAIN_CONSOLA, "Imposible conectar con el robot.");
	  } else {
	       SetCtrlVal(panel_control, MAIN_CONSOLA, "Parado.");
	  }
	  break;
	case MAIN_DIRLOCAL:     /* establecer nueva direccion local de PC */
	  GetCtrlVal(panel_control, MAIN_DIRLOCAL, &temp);
	  setDirLocal(temp);
	  SetCtrlVal(panel_control, MAIN_CONSOLA, "Establecida nueva direccion local del PC.");
	  break;
	case MAIN_PASOS:        /* solicitar numero de pasos restantes para finalizar el movimiento */
	  SetCtrlVal(panel_control, MAIN_CONSOLA, "Solicitando numero de pasos restantes.");
	  temp=pasos(robot,&numPasos);
	  if (temp==2) {
	       MessagePopup("Fallo en la conexion con el modem.");
	  } else if (temp==1) {
	       SetCtrlVal(panel_control, MAIN_CONSOLA, "Imposible conectar con el robot.");
	  } else {
	       SetCtrlVal(panel_control, MAIN_CONSOLA, "Recibido numero de pasos.");
	       SetCtrlVal(panel_control, MAIN_NUMPASOS, numPasos);
	  }
	  break;
      case MAIN_CONVER:        /* solicitar conversion A/D */
	  GetCtrlVal(panel_control, MAIN_CANAL, &temp);
	  SetCtrlVal(panel_control, MAIN_CONSOLA, "Solicitando conversion A/D.");
	  temp2=canalAnalog(robot,temp,&dato);
	  if (temp2==2) {
	       MessagePopup("Fallo en la conexion con el modem.");
	  } else if (temp2==1) {
	       SetCtrlVal(panel_control, MAIN_CONSOLA, "Imposible conectar con el robot.");
	  } else {
	       SetCtrlVal(panel_control, MAIN_CONSOLA, "Recibida conversion A/D.");
	       SetCtrlVal(panel_control, MAIN_VALOR, dato);
	  }
	  break;
      case MAIN_PORT:          /* solicitar lectura de un puerto del Microbot */
	  GetCtrlVal(panel_control, MAIN_PUERTO, &temp);
	  SetCtrlVal(panel_control, MAIN_CONSOLA, "Solicitando lectura del PUERTO.");
	  temp2=puerto(robot,temp,&dato);
	  if (temp2==2) {
	       MessagePopup("Fallo en la conexion con el modem.");
	  } else if (temp2==1) {
	       SetCtrlVal(panel_control, MAIN_CONSOLA, "Imposible conectar con el robot.");
	  } else {          // se actualizan los indicadores
	       SetCtrlVal(panel_control, MAIN_LECTURA, dato);
	       SetCtrlVal(panel_control, MAIN_BIT0,dato&0X01);
	       SetCtrlVal(panel_control, MAIN_BIT1,dato&0X02);
	       SetCtrlVal(panel_control, MAIN_BIT2,dato&0X04);
	       SetCtrlVal(panel_control, MAIN_BIT3,dato&0X08);
	       SetCtrlVal(panel_control, MAIN_BIT4,dato&0X10);
	       SetCtrlVal(panel_control, MAIN_BIT5,dato&0X20);
	       SetCtrlVal(panel_control, MAIN_BIT6,dato&0X40);
	       SetCtrlVal(panel_control, MAIN_BIT7,dato&0X80);
	       SetCtrlVal(panel_control, MAIN_CONSOLA, "Dato recibido satisfactoriamente.");
	  }
	  break;
       }
   }
}

/*-----------------------Funciones----------------------------*/

void dibujarPunto(short int sentido, unsigned short int pasos)
{                        //Funcion para desplazar la posicion del robot en la grafica tras un avance o retroceso
   unsigned short int i;
   short int mod;
   double angulo;
   angulo = ALPHA * dir;	// 1-->Adelante; -1-->Atras
   mod = MODULO * sentido;
   for (i=0;i<pasos;i++) {      // Se dibuja la estela del movimiento del robot
     PlotPoint (panel_control, MAIN_GRAPH, x, y, 0, 2);
     x = x + ( mod * sin(angulo) );
     y = y + ( mod * cos(angulo) );
     if (x >= DIMGRAPH) {          // Detectamos si se superan los limites de la grafica
	 x=0.0;
	 DeletePlots (panel_control, MAIN_GRAPH);
     }else if (x<=0.0) {
	 x=DIMGRAPH;
	 DeletePlots (panel_control, MAIN_GRAPH);
     }
     if (y>=DIMGRAPH) {
	 y=0.0;
	 DeletePlots (panel_control, MAIN_GRAPH);
     }else if (y<=0.0) {
	 y=DIMGRAPH;
	 DeletePlots (panel_control, MAIN_GRAPH);
     }
     PlotPoint (panel_control, MAIN_GRAPH, x, y, 0, 1);  // Se pinta la posicion final del robot
   }
}

void girarPunto(short int sentido, unsigned short int pasos)  //Funcion para establecer el nuevo rumbo del robot tras un giro
{                         //Funcion para establecer el nuevo rumbo del robot tras un giro
   unsigned short int i;
   double angulo;
   for (i=0;i<pasos;i++) {
	dir = dir+sentido;       // 1-->Derecha; -1-->Izquierda
	if (dir==-1) {		 // dir = 0 --> hacia arriba
	    dir=P2PI-1;
	}else if (dir==P2PI) {   // 360 grados / 11.25 grados = 32 direcciones posibles
	    dir=0;
	}
	SetCtrlVal(panel_control, MAIN_RUMBO, ALPHAG*dir);    // Se actualiza el rumbo
   }
}