// protocol.h
/* Libreria que incluye las funciones que definen el
protocolo de comunicacion entre el PC y el microbot */

/*-----------------Librerias incluidas--------------------------------------*/

#include "com.h"

/*-----------------Definicion de Constantes---------------------------------*/

#define NUMREP  100  // Numero de veces que repite la transmision de un dato hasta obtener respuesta
#define ADELANTE  0
#define IZQUIERDA 1
#define DERECHA   2
#define ATRAS     3
#define PARO      4
#define PORTA     0
#define PORTB     1
#define PORTC     2
#define PORTD     3
#define PORTE     4

/*--------------------Variables globales----------------------------------*/

unsigned short int mask[16]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};  // Mascaras que determinan el actual bit de control para cada Microbot

/*-----------------Declaracion Funciones ---------------------------------*/

unsigned short int avanzar(unsigned short int robot, unsigned short int n);
unsigned short int retroceder(unsigned short int robot, unsigned short int n);
unsigned short int girarDerecha(unsigned short int robot, unsigned short int n);
unsigned short int girarIzquierda(unsigned short int robot, unsigned short int n);
unsigned short int parar(unsigned short int robot);
unsigned short int pasos(unsigned short int robot, unsigned short int *dato);
unsigned short int puerto(unsigned short int robot, unsigned short int puerto, unsigned short int *dato);
unsigned short int canalAnalog(unsigned short int robot, unsigned short int canal, unsigned short int *dato);

unsigned short int comandoMov(unsigned short int robot, unsigned short int pasos, unsigned short int dir);
unsigned short int peticion(unsigned short int robot, unsigned short int option, unsigned short int num, unsigned short int *dato);
unsigned short int changeMask(unsigned short int robot);

/*----------------Implementacion Funciones------------------------------------------*/

/*-- Funciones para ordenar movimientos al robot (llaman a la funcion ComandoMov) --*/

unsigned short int avanzar(unsigned short int robot, unsigned short int n)
{
 return(comandoMov(robot,n,ADELANTE));
}
unsigned short int retroceder(unsigned short int robot, unsigned short int n)
{
 return(comandoMov(robot,n,ATRAS));
}
unsigned short int girarDerecha(unsigned short int robot, unsigned short int n)
{
 return(comandoMov(robot,n,DERECHA));
}
unsigned short int girarIzquierda(unsigned short int robot, unsigned short int n)
{
 return(comandoMov(robot,n,IZQUIERDA));
}

/*-- Funcion para solicitar al Microbot la lectura de un puerto --*/

unsigned short int puerto(unsigned short int robot, unsigned short int puerto, unsigned short int *dato)
{
	return(peticion(robot,0,puerto,dato));
}

/*-- Funcion para solicitar al Microbot una conversion A/D --*/

unsigned short int canalAnalog(unsigned short int robot, unsigned short int canal, unsigned short int *dato)
{
	return(peticion(robot,1,canal,dato));
}

/*-- Funcion para cambiar el BIT de Control para las tarnsmisiones a un Microbot --*/

unsigned short changeMask(unsigned short int robot)
{
	if (mask[robot]==0x00) {
	    mask[robot]=0x08;
	} else {
	    mask[robot]=0x00;
	}
	return(TRUE);
}

/*-- Funcion para ordenar a un Microbot que se pare --*/

unsigned short int parar(unsigned short int robot)
{
	unsigned short int acierto,aciertoModem,i,d;
	acierto=FALSE;
	i=0;
	do {
		aciertoModem=enviarNibble(robot,mask[robot] | 0x4);		// envia dato
		if (aciertoModem==FALSE) { break; }
		if ( (recibirNibble(&d)==TRUE) ) { //&& (d==mask[robot] | 0x4) ) {
			acierto=TRUE;
		}
		i++;
	} while ( (acierto==FALSE) && (i<NUMREP) );
	changeMask(robot);			 // Cambia BIT de control
	if (aciertoModem==FALSE) { return(2); }  // Retorna que ha fallado la conexion con el modem
	else if (acierto==FALSE) { return(1); }  // Retorna que no se ha podido completar la comunicacion con el robot
	else { return(0); }                      // Retorna que la comunicacion se ha completado satisfactorimente
}

/*-- Funcion para ordenar un movimiento a un Microbot --*/

unsigned short int comandoMov(unsigned short int robot, unsigned short int pasos, unsigned short int dir)
{
	unsigned short int d,acierto,aciertoModem,aciertoParar,i,j;
	unsigned short int nibble[3];
	if (dir==PARO) {
	    return(parar(robot));
	}
	nibble[0]=dir;
	nibble[2]=(pasos & 0x07);
	nibble[1]=( (pasos>>3) & 0x07 );
	i=0;
	aciertoParar=0;
	do {
		j=0;
		do {
			aciertoModem=enviarNibble( robot , mask[robot] | nibble[i] );
			if (aciertoModem==FALSE) { break; }
			if ( (recibirNibble(&d)==TRUE) ) { //& (d==mask[robot] | nibble[i]) ) {
				acierto=TRUE;
			} else {
				acierto=FALSE;
			}
			j++;
		} while ( (acierto==FALSE) && (j<NUMREP) );
		changeMask(robot);
		i++;
	} while ( (acierto==TRUE) && (i<3) && (aciertoModem==TRUE) );
	if ((aciertoModem==FALSE)||(aciertoParar==2)) { return(2);
	} else if (acierto==FALSE) { return(1);
	} else { return(0); }
}

/*-- Funcion para solicitar al robot el numero de pasos restantes para finalizar el ultimo movimiento --*/

unsigned short int pasos(unsigned short int robot, unsigned short int *dato)
{
	unsigned short int i,j,acierto,d,aciertoModem;
	unsigned short int nibbleOut[2];
	unsigned short int nibbleIn[2];
	nibbleOut[0]=0x05;
	nibbleOut[1]=0x00;
	i=0;
	do {
		j=0;
		do {
			aciertoModem=enviarNibble( robot , mask[robot] | nibbleOut[i] );
			if (aciertoModem==FALSE) { break; }
			if (recibirNibble(&d)==FALSE) {
				acierto=FALSE;
			} else {
				acierto=TRUE;
				if(i==0) { nibbleIn[1]=d;            // Almacena el Nibble alto
				} else if (i==1) { nibbleIn[0]=d; }  // Almacena el Nibble bajo
			}
			j++;
		} while ( (acierto==FALSE) && (j<NUMREP) );
		changeMask(robot);
		i++;
	} while ( (acierto==TRUE) && (i<2) && (aciertoModem==TRUE) );
	if (aciertoModem==FALSE) { return(2);
	} else if (acierto==FALSE) {
		return(1);
	} else {
		*dato=( (nibbleIn[1]<<4) | (nibbleIn[0]) );
		return(0);
	}
}

/*--- Funcion para solicitar al robot la lectura de un puerto o una conversion A/D --*/

unsigned short int peticion(unsigned short int robot, unsigned short int option, unsigned short int num, unsigned short int *dato)
{
	unsigned short int i,j,acierto,d,aciertoModem;
	unsigned short int nibbleOut[3];
	unsigned short int nibbleIn[2];
	nibbleOut[0]=0x06+option;		// Puerto o conversion A/D
	nibbleOut[1]=num;
	nibbleOut[2]=0x00;
	i=0;
	do {
		j=0;
		do {
			aciertoModem=enviarNibble( robot , mask[robot] | nibbleOut[i] );
			if (aciertoModem==FALSE) { break; }
			if ( (recibirNibble(&d)==FALSE) ) {  //| ( (i==0) && (d!=mask[robot] | nibbleOut[i]) ) ) {
				acierto=FALSE;
			} else {
				acierto=TRUE;
				if(i==1) { nibbleIn[1]=d;             // Almacena el Nibble alto
				} else if (i==2) { nibbleIn[0]=d; }   // Almacena el Nibble bajo
			}
			j++;
		} while ( (acierto==FALSE) && (j<NUMREP) );
		changeMask(robot);
		i++;
	} while ( (acierto==TRUE) && (i<3) && (aciertoModem==TRUE) );
	if (aciertoModem==FALSE) { return(2);
	} else if (acierto==FALSE) {
		return(1);
	} else {
		*dato=( (nibbleIn[1]<<4) | (nibbleIn[0]) );
		return(0);
	}
}