;PROYECTO.ASM
;Programa para el control de un microbot mediante un enlace radio bidireccional

OPTION  EQU $39
PIOC    EQU $02
PORTA   EQU $00
PORTB   EQU $04
PORTC   EQU $03
PORTD   EQU $08
DDRD    EQU $09
PORTE   EQU $0A
SCCR2   EQU $2D
SPCR    EQU $28
HPRIO   EQU $3C
TMSK1   EQU $22
TFLG1   EQU $23
TMSK2   EQU $24
TFLG2   EQU $25
PACTL   EQU $26
PACNT   EQU $27
TCTL1   EQU $20
TOC4    EQU $1C
TCNT    EQU $0E
ADCTL   EQU $30
ADR1    EQU $31
DIRPC   EQU $00
TIEMPO  EQU 65000  ; Numero de pulsos de reloj necesarios para generar un
                   ; retraso 32,5 ms. Cada tic de reloj son 0.5 us
                   ; 65000*0.5 = 32500 us = 32,5 ms.
;Variables

        ORG $0000        ; RAM

dato_recibido  RMB 1     ; Variables que no se inicializan, solo se reserva un Byte
dato           RMB 1
mask           RMB 1
temp           RMB 1

;Programa

        ORG $B600        ; EEPROM

config  LDX #$1000
        LDS #$00B4       ; DIR 180 Comienzo de al pila
        
        LDAA #$00
        STAA  SCCR2,X    ; SCI deshabilitado
        
        LDAA #$30
        STAA  SPCR,X     ; SPI deshabilitado y salidas OC 
        STAA  DDRD,X     ; BITS 0 a 3 del puerto D entradas, BITS 4 y 5 salidas

        LDAA #$00
        STAA TMSK1,X     ; No permitir la interupcin de los comparadores
        STAA PACTL,X     ; Bit 7 Puerto A entrada, ac de pulsos desactivado
        STAA dato_recibido
        
        LDAA  #$20
        STAA  PORTD,X    ; BIT 4 a 0 y BIT 5 del puerto D a 1
        STAA OPTION,X    ; IRQ activa por flanco de bajada

        LDAA #$C6
        STAA HPRIO,X     ; Maxima prioridad IRQ

        LDAA #$7E        ; cargamos en el A el codigo de la instruccion JUMP
        STAA $EE         ; principio del vector de interrupcion de la IRQ
        LDD  #int_irq    ; D <- direccion de comienzo de la rutina interrup.
        STD  $EF

        LDAA #$7E        ; cargamos en el A el codigo de la instruccion JUMP
        STAA $CD         ; principio del vector de interrupcion del acumulador
        LDD  #int_ac     ; D <- direccion de comienzo de la rutina interrup.
        STD  $CE

        CLI              ; permite interrupciones

        JSR recibir      ; espera a recbir el primer dato
        JSR unmask       ; obtiene el bit de control del dato recibido
inicio  STAA mask        ; actualiza el bit de control
        CMPB #$04        ; evalua el comando recibido
        BNE c5
        JMP parar
c5      CMPB #$05        ; peticion numero de pasos restantes
        BNE c6
        JMP pasos         
c6      CMPB #$06        ; peticion lectura de un puerto
        BNE c7
        JMP puerto
c7      CMPB #$07        ; peticion conversion A/D
        BNE mover        ; si no es ninguna de las anteriores --> Comando movimiento
        JMP analog
        
mover   ORAB mask        ; enmascara el dato aadiendole el bit de control 
        JSR enviar       ; lo envia (es un eco)
        ;ANDB #$03        ; me quedo con los dos bits menos significativos (redundante)
        LSLB             ; Desplazo el dato recibido 5 bits a la izquierda
        LSLB
        LSLB
        LSLB
        LSLB
        STAB PORTA,X     ; Si Bits 6 y 5 = 00-->delante ; 11-->detras ; 10-->Derecha ; 01--> Izquierda
em1     JSR recibir      ; espera parte alta del numero de pasos
        JSR enviar       ; envia un eco
        JSR unmask
        CMPA mask        ; comprueba si ha variado el bit de control en el nuevo dato recibido
        BEQ em1          ; si no ha variado vuelve a la espera anterior
        STAA mask        ; actualiza bit de control

        LSLB             ; desplaza el dato 3 bits a la izquierda
        LSLB
        LSLB
        STAB dato        ; almacena los 3 bits mas significativos del numero de pasos
em2     JSR recibir      ; espera parte baja del numero de pasos
        JSR enviar
        JSR unmask
        CMPA mask        ; aade al dato el bit de control
        BEQ em2
        STAA mask        ; actualiza bit de control

        ORAB dato
        STAB dato
        CMPB #$00
        BEQ sinfin       ; Si pasos==0 --> Movimiento sin fin
        NEGB
        ADDB #$FF
        STAB PACNT,X     ; Carga el numero de pasos en el acumulador de pulsos
        BSET TMSK2,X $20 ; Activar interrupcion de overflow del acumulador
        BSET PACTL,X $40 ; Activa el acumulador de pulsos
sinfin  BSET PORTA,X $18 ; Enciende los motores
em3     JSR recibir      ; espera una nueva recepcion
        JSR unmask
        CMPA mask
        BNE fin_mov      ; si es un nuevo comando termina el movimiento
        ORAB mask
        JSR enviar       ; si no envia un eco
        BRA em3          ; y vuelve atras
fin_mov JMP inicio

parar   JSR detener      ; para el Microbot
        ORAB mask        ; aade bit control
        JSR enviar       ; envia eco
        JSR recibir      ; espera una nueva recepcion
        JSR unmask
        CMPA mask        
        BEQ parar        ; si no es un nuevo comando vuelve atras
        JMP inicio

pasos   LDAB #$00
        BRCLR PACTL,X $40 store ; si el robot esta parado devuelve 0
        LDAB #$FF        
        SUBB PACNT,X            ; si no devuelve el numero de pasos restantes
store   STAB dato
        JSR des4der               
        JSR enviar              ; Envia el nibble alto
        JSR recibir             ; Espero al siguiente dato
        JSR unmask
        CMPA mask
        BEQ pasos               ; si no cambia el bit de control vuelve atras
        STAA mask
ev2     LDAB dato        
        JSR enviar              ; Envia el nibble bajo
        JSR recibir             ; espera una nueva recepcion
        JSR unmask
        CMPA mask
        BEQ ev2                 ; si no es un nuevo comando vuelve atras
        JMP inicio

puerto  ORAB mask               ; aade el bit de control
        JSR enviar              ; hace eco
        JSR recibir             ; espera a recibir el puerto
        JSR unmask
        CMPA mask
        BEQ puerto              ; si el bit de control no cambia vuelve atras
        STAA mask
ep2     CMPB #$00               ; lee el puerto correspondiente
        BNE cp1
        LDAB PORTA,X
        BRA fc
cp1     CMPB #$01
        BNE cp2
        LDAB PORTB,X
        BRA fc
cp2     CMPB #$02
        BNE cp3
        LDAB PORTC,X
        BRA fc
cp3     CMPB #$03
        BNE cp4
        LDAB PORTD,X
        BRA fc
cp4     BCLR OPTION,X $80       ;apaga el conversor A/D 
        LDAB PORTE,X
fc      STAB dato
        JSR des4der
        JSR enviar              ; envia el nibble alto
        JSR recibir             ; espero a la siguiente recepcion
        JSR unmask
        CMPA mask
        BEQ ep2                 ; si el bit de control no cambia vuelve atras
        STAA mask
	BRA e3

analog  BSET OPTION,X $80       ; encender el conversor A/D
        ORAB mask
        JSR enviar
        JSR recibir             ; espera a recibir el canal
        JSR unmask
        CMPA mask
        BEQ analog              ; si el bit de control no cambia vuelve atras
        STAA mask
        ;ANDB #$07               ; nos quedamos con el numero de canal (redundante)
        STAB ADCTL,X            ; SCAN -> inactivo ; MULT -> inactivo ; ADR1 -> seleccionar primer canal 
sigue   BRCLR ADCTL,X $80 sigue ; espera a que termine conversion
ea2     LDAB ADR1,X             ; lee el resultado de la conversion
        STAB dato
        JSR des4der
        JSR enviar              ; Envia el nibble alto
        JSR recibir             ; Espero al siguiente nibble
        JSR unmask
        CMPA mask
        BEQ ea2                 ; si el bit de control no cambia vuelve atras
        STAA mask

e3      LDAB dato
        JSR enviar              ; envia el nibble bajo
        JSR recibir             ; espera una nueva recepcion
        JSR unmask
        CMPA mask
        BEQ e3                  ; si el bit de control no cambia vuelve atras
        JMP inicio

;funciones

enviar  BRSET PORTD,X $10 enviar  ; Subrutina que transmite el dato en el acum B

        STAB temp

        BSR delay                 ; Espera 32,5 ms

        BSET DDRD,X $0F           ; Bits 0 a 3 del PUERTO D salidas

        LDAA #DIRPC	
        ;ANDA #$0F
        ORAA #$10
        STAA PORTD,X              ; Escribe la direccion destino e inicia la transmision

        BSET PORTD,X $20
        BSET PORTD,X $20          ; Se repite para que transcurra el tiempo thl
        BSET PORTD,X $20
        BSET PORTD,X $20
                
        LDAB temp
        ORAB #$30
        STAB PORTD,X              ; Escribe el dato a transmitir

        ;ANDA #$0F
        ANDB #$0F
esp_tx  BRSET PORTD,X $10 esp_tx  ; Espera a que finalice la transmision
        RTS

recibir LDAB dato_recibido        ; Subrutina que espera a que se reciba un dato para leerlo
        CMPB #$00
        BEQ recibir               ; Espera a que se reciba un dato
;espera  BRSET PORTD,X $10 espera ; Espera a que finalice una posible transmision    
        LDAB #$00
        STAB dato_recibido        ; indica que se ha leido el dato   
        LDAB PORTD,X              ; lee el dato y lo deja en el acum B
        ANDB #$0F
        RTS

delay   LDD TCNT,X                 ; Subrutina que espera 32 ms
        ADDD #TIEMPO               ; Esperar 32 ms
        STD TOC4,X		   ; Escribe en el comparador 4
        BSET TFLG1,X $10           ; Poner a cero flag del comparador 4
buclees BRCLR TFLG1,X $10 buclees  ; Comprobar si a terminado la cuenta
        RTS

unmask  TBA           ; Subrutina que separa el BIT de control del dato recibido 
        ANDB #$07     ; dato en acum B
        ANDA #$08     ; mascara para el BIT de control en el acum A
        RTS             

des4der LSRB          ; Subrutina que desplaza el dato en el acum B 4 bits a la derecha
        LSRB
        LSRB
        LSRB
        RTS

detener BCLR PORTA,X $18     ; Parar motores
        BCLR TMSK2,X $20     ; Desactiva interrupcion de overflow del acumulador
        BCLR PACTL,X $40     ; Desactiva el acumulador de pulsos
        RTS

;Rutina de atencion a la interrupcion externa IRQ

int_irq BRSET PORTD,X $10 fin_tr ; Si se estaba transmitiendo salta a finalizar la transmision
        LDAA #$01
        STAA dato_recibido       ; Si no se estaba transmitiendo activa dato_recibido
        RTI
fin_tr  BCLR DDRD,X $0F          ; Bits 0 a 3 del PUERTO D entradas
        BCLR PORTD,X $10         ; Pone la tarjeta en recepcion
        RTI

;Rutina de atencion a la interrupcion de overflow del acumulador de pulsos

int_ac  BSET TFLG2,X $20 ; Poner a cero flag de overflow del ac
        BSR detener	 ; Para el Microbot
        RTI

	END
