//prueba.c
/* Programa para probar en funcionamiento del enlace radio bidireccional
diseado para comunicar un PC con distintos Microbots */

/*--------------------- Librerias incluidas --------------------*/

#include "com.h"
#include <stdio.h>
#include <bios.h>
#include <dos.h>

/*--------------------- Funciones ------------------------------*/

void prueba_trans()
{       // Prueba para comprobar que el modem transmite bien todos los datos posibles
	unsigned short int i,a,error;
	unsigned int fallos,aciertos,total;
	double t1,t2,t3,t4,t;
	clrscr();
	fallos=0;
	aciertos=0;
	total=0;
	t3=clock();
	for (a = 0; a <= 15; a++) {  // tramsmite todos los datos posibles a todis las dirrecciones posibles
	 for (i = 0; i <= 15; i++) {
		error=FALSE;
		t1=clock();
		enviar(a,i);
		while ( (transmitiendo==TRUE) && (error==FALSE) )  // Se espera a que ternmine la transmision
		{
			t2=clock();
			t=(t2 - t1)/CLK_TCK;
			if (t > TETX) {    // Se pone el modem en recepcion y se configura su direccion local
				transmitiendo=FALSE;
				dato_recibido=FALSE;
				outportb( DATA , (inportb(DATA) & 0x0F) | (dir_local<<4) );
				outportb(CONTROL,0x12);
				error=TRUE;   // No se detecto el final de la transmision
			}
		}
		printf("dir:%d  dato:%2d   ",a,i);
		if (error==TRUE) {   // No se ha detectado el fin de la transmision
			printf("Tiempo sobrepasado.");
			fallos++;
		}
		else if (recibir()==i) {   // La transmision se completo satisfactoriamente
			printf("Prueba satisfactoria.");
			aciertos++;
		}
		else {                     // No se transmitio el dato correctamente
			printf("Prueba no satisfactoria.");
			fallos++;
		}
		printf(" Duracion: %f milisegundos.\n",t*1000.0); // Se muestra la duracion de la transmision
	 }
	}
	t4=(t2-t3)/CLK_TCK;
	total=fallos+aciertos;
	printf("%d Transmisiones en total.\n",total);      // Se muestra una estadistica de los resultados
	printf("%f Segundos en total.\n",t4);
	t3=(t4*1000.0)/total;
	printf("%f Milisegundos de media cada transmision.\n",t3);
	t3=1000/t3;
	printf("%f Transmisiones por segundo.\n",t3);
	printf("%1d fallos, %2d aciertos.\n",fallos,aciertos);
	printf("%1f por ciento de fallos, %2f de aciertos.\n",(fallos*100.0)/total,(100.0*aciertos)/total);
	printf("Pulse una tecla para continuar.");
	while (kbhit()==FALSE) {
	}
}

void prueba_recep()    // Prueba que muestran todos los datos que se reciben
{
	unsigned short int dato,dir;
	double t1,t2;
        char ch;
	ch='0';
	t1=0;
        t2=0;
	clrscr();
	printf("Introduzca la direccion local del PC:  ");
	scanf("%u",&dir);
	printf("\n");
	setDirLocal(dir);  // Se configura el modem con la direccion local indicada por el usuario
	outportb(CONTROL,0x12); // Se configura el modem en Recepcion
	printf("Prueba de recepcion. Pulsar ESC para salir \n");
	do {
		while ((datoListo()==FALSE) & (ch !=27))
		{
			if (kbhit()){
			ch = getch();
			}
		}
		if (datoListo()==TRUE) {   // Se ha recibido un dato
			t2=clock();
			dato=recibir();    // Se lee el dato
			printf("dato recibido: %d   ",dato);  // Se muestra el dato recibido
			printf("tiempo desde la ultima recepcion: %f \n",(t2-t1)/CLK_TCK);
			t1=t2;
                }
	} while (ch !=27); // Se sale cuando se pulsa ESC (ASCII 27)

}

void prueba_eco()
{       // Para probar la bidireccionalidad del enlace. Para cada dato transmitido se espera un eco
	unsigned short int i,j,numit,dir,dato,error;
	unsigned int fallos,aciertos,total;
	double t1,t2,t3,t4;
	clrscr();
	fallos=0;
	aciertos=0;
	total=0;
	printf("Introduzca el numero de repeticiones de la prueba:  ");
	scanf("%u",&numit);
	printf("\n");
	printf("Introduzca la direccion local del PC:  ");
	scanf("%u",&dir);
	printf("\n");
	setDirLocal(dir); // Se configura el modem con la direccion local indicada por el usuario
	printf("Introduzca la direccion del robot:  "); // Se pide al usuario la direccion destino
	scanf("%u",&dir);
	printf("\n");
	t3=clock();
	for (j = 1; j <= numit; j++) { // Se envia ciclicamente del 0 al 15 esperando un eco para cada uno
	for (i = 0; i <= 15; i++) {
		printf("Enviando: dir:%d  dato:%2d   ",dir,i);
		enviarNibble(dir,i);    // Se envia el dato
		error=FALSE;
		t1=clock();
		while ( (datoListo()==FALSE) && (error==FALSE) )   // Se espera a recibir el eco
		{
			t2=clock();
			if ( ((t2-t1)/CLK_TCK) > TERX) {
				error=TRUE;    // No se recibio el eco
			}
		}
		if (error==TRUE) {         // No se recibio el eco
			printf("Tiempo para el eco sobrepasado.\n");
			fallos++;
		}
		else {
			dato=recibir();
			printf("Eco recibido:%d  ",dato);
			if (dato==i) {      // Se recibio un eco correcto
				printf("==> Eco satisfactorio.\n");
				aciertos++;
			}
			else {              // No se recibio un eco correcto
				printf("==> Eco no satisfactorio.\n");
				fallos++;
			}
		}
	}
	}
	t4=clock();
	t4=(t4-t3)/CLK_TCK;
	total=fallos+aciertos;
	printf("%d Transmisiones en total.\n",total);     // Se muestra una estadistica
	printf("%f Segundos en total.\n",t4);
	t3=(t4*1000.0)/total;
	printf("%f Milisegundos cada transmision y su eco.\n",t3);
	t3=1000/t3;
	printf("%f Transmisiones y ecos por segundo.\n",t3);
	printf("%1d fallos, %2d aciertos.\n",fallos,aciertos);
	printf("%1f por ciento de fallos, %2f de aciertos.\n",(fallos*100.0)/total,(100.0*aciertos)/total);
	printf("Pulse una tecla para continuar.");
	while (kbhit()==FALSE) {
	}
}

void main()
{
	char *elec;
	*elec='0';
	inicializar();          // Se inicializa el Modem
	while (*elec!='4') {
		clrscr();       // Se muestra el menu
		printf("1. Prueba de transmision. \n");
		printf("2. Prueba de recepcion. \n");
		printf("3. Prueba de eco. \n");
		printf("4. Salir. \n");
		printf("Elija un opcion: ");
		scanf("%c",elec);
		if (*elec=='1') {
			prueba_trans();       // Prueba de transmision
		}
		else if (*elec=='2') {
			prueba_recep();       // Prueba de recepcion
		}
		else if (*elec=='3') {
			prueba_eco();         // Prueba de eco
		}
	}
	terminar();
}